searchData={"items":[{"type":"task","title":"mix firmware.gen.script","doc":"Creates a shell script for invoking ssh to upgrade devices with ssh_subsystem_fwup.\n\nThis script may be used on its own or used as a base for more complicated\ndevice software upgrade deployments.\n\nIt saves the script to upload.sh.","ref":"Mix.Tasks.Firmware.Gen.Script.html"},{"type":"function","title":"Mix.Tasks.Firmware.Gen.Script.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Firmware.Gen.Script.html#run/1"},{"type":"task","title":"mix upload","doc":"Upgrade the firmware on a Nerves device using SSH.\n\nBy default, `mix upload` reads the firmware built by the current `MIX_ENV`\nand `MIX_TARGET` settings, and sends it to `nerves.local`. Pass in a another\nhostname to send the firmware elsewhere.\n\nNOTE: This implementation cannot ask for passphrases, and therefore, cannot\nconnect to devices protected by username/passwords or decrypt\npassword-protected private keys. One workaround is to use the `ssh-agent` to\npass credentials.","ref":"Mix.Tasks.Upload.html"},{"type":"task","title":"Command line options - mix upload","doc":"* `--firmware` - The path to a fw file","ref":"Mix.Tasks.Upload.html#module-command-line-options"},{"type":"task","title":"Examples - mix upload","doc":"Upgrade a Raspberry Pi Zero at `nerves.local`:\n\n    MIX_TARGET=rpi0 mix upload nerves.local\n\nUpgrade `192.168.1.120` and explicitly pass the `.fw` file:\n\n    mix upload 192.168.1.120 --firmware _build/rpi0_prod/nerves/images/app.fw","ref":"Mix.Tasks.Upload.html#module-examples"},{"type":"module","title":"SSHSubsystemFwup","doc":"SSH subsystem for upgrading Nerves devices\n\nThis module provides an SSH subsystem for Erlang's `ssh` application. This\nmakes it possible to send firmware updates to Nerves devices using plain old\n`ssh` like this:\n\n```shell\ncat $firmware | ssh -s $ip_address fwup\n```\n\nWhere `$ip_address` is the IP address of your Nerves device. Depending on how\nyou have Erlang's `ssh` application set up, you may need to pass more\nparameters (like username, port, identities, etc.).\n\nSee [`nerves_ssh`](https://github.com/nerves-project/nerves_ssh/) for an easy\nway to set this up. If you don't want to use `nerves_ssh`, then in your call\nto `:ssh.daemon` add the return value from\n`SSHSubsystemFwup.subsystem_spec/1`:\n\n```elixir\ndevpath = Nerves.Runtime.KV.get(\"nerves_fw_devpath\")\n\n:ssh.daemon([\n      {:subsystems, [SSHSubsystemFwup.subsystem_spec(devpath: devpath)]}\n    ])\n```\n\nSee `SSHSubsystemFwup.subsystem_spec/1` for options. You will almost always\nneed to pass the path to the device that should be updated since that is\ndevice-specific.","ref":"SSHSubsystemFwup.html"},{"type":"type","title":"SSHSubsystemFwup.options/0","doc":"Options:\n\n* `:devpath` - path for fwup to upgrade (Required)\n* `:fwup_path` - path to the fwup firmware update utility\n* `:fwup_env` - a list of name,value tuples to be passed to the OS environment for fwup\n* `:fwup_extra_options` - additional options to pass to fwup like for setting\n  public keys\n* `:precheck_callback` - an MFArgs to call when there's a connection. If\n  specified, the callback will be passed the username and the current set of\n  options. If allowed, it should return `{:ok, new_options}`. Any other\n  return value closes the connection.\n* `:success_callback` - an MFArgs to call when a firmware update completes\n  successfully. Defaults to `{Nerves.Runtime, :reboot, []}`.\n* `:task` - the task to run in the firmware update. Defaults to `\"upgrade\"`","ref":"SSHSubsystemFwup.html#t:options/0"},{"type":"function","title":"SSHSubsystemFwup.subsystem_spec/1","doc":"Helper for creating the SSH subsystem spec","ref":"SSHSubsystemFwup.html#subsystem_spec/1"},{"type":"extras","title":"SSHSubsystemFwup","doc":"# SSHSubsystemFwup\n\n[![Hex version](https://img.shields.io/hexpm/v/ssh_subsystem_fwup.svg \"Hex version\")](https://hex.pm/packages/ssh_subsystem_fwup)\n[![API docs](https://img.shields.io/hexpm/v/ssh_subsystem_fwup.svg?label=hexdocs \"API docs\")](https://hexdocs.pm/ssh_subsystem_fwup/SSHSubsystemFwup.html)\n[![CircleCI](https://dl.circleci.com/status-badge/img/gh/nerves-project/ssh_subsystem_fwup/tree/main.svg?style=svg)](https://dl.circleci.com/status-badge/redirect/gh/nerves-project/ssh_subsystem_fwup/tree/main)\n[![REUSE status](https://api.reuse.software/badge/github.com/nerves-project/ssh_subsystem_fwup)](https://api.reuse.software/info/github.com/nerves-project/ssh_subsystem_fwup)\n\nThis library provides an [ssh](https://en.wikipedia.org/wiki/Secure_Shell)\nsubsystem that applies Nerves \"over-the-air\" firmware updates. It is an\nalternative to\n[`nerves_firmware_ssh`](https://github.com/nerves-project/nerves_firmware_ssh)\nthat extracts the update service to a `:ssh.daemon/1` spec. This trims down the\nresponsibilities of the library and makes it possible to:\n\n1. Customize ssh authentication (for example, password-based auth is possible)\n2. Handle host keys differently and more securely\n3. Run firmware updates on port 22 with other ssh services\n\nIn addition, the protocol for sending updates over ssh has been simplified. If\nyou're coming from `nerves_firmware_ssh`, you'll have used the `upload.sh`\nscript or `mix upload`. This library provides the same interface. If using\n`upload.sh`, you will need to rerun `mix firmware.gen.script` since the script\nhas changed.","ref":"readme.html"},{"type":"extras","title":"Installation - SSHSubsystemFwup","doc":"The easiest installation is to use\n[`nerves_ssh`](https://github.com/nerves-project/nerves_ssh) and have it bring\nin this library as a dependency. See that project for details.\n\nHowever, if you do not want to use `nerves_ssh`, here's what do do. First, add\nthe dependency:\n\n```elixir\ndef deps do\n  [{:ssh_subsystem_fwup, \"~> 0.6.0\"}]\nend\n```\n\nThen add ssh subsystem spec to the call that starts the `ssh` daemon. This code\nwill look something like:\n\n```elixir\n    {:ok, ref} =\n      :ssh.daemon([\n        {:subsystems, [SSHSubsystemFwup.subsystem_spec()]}\n      ])\n```\n\nYou will likely have many more options passed to the `ssh.daemon`.","ref":"readme.html#installation"},{"type":"extras","title":"Uploading firmware - SSHSubsystemFwup","doc":"There are two ways of uploading firmware. The first is to run:\n\n```shell\nmix upload\n```\n\nThat doesn't work for everyone due to `ssh` authentication preferences. The\nalternative is to use commandline `ssh`. For convenience, `ssh_subsystem_fwup`\ncan generate a script that makes this easier. Go to your Nerves project\ndirectory and run:\n\n```shell\nmix firmware.gen.script\n```\n\nThis should create an `upload.sh` script. Frequently when starting out, you can\nrun `./upload.sh` without arguments since it will guess that it's supposed to\nupload to `nerves.local`. To specify a device to upload to, pass the device's\nhostname as the first argument. For example:\n\n```shell\n$ ./upload.sh nerves-1234.local\nfwup: Upgrading partition B\n|====================================| 100% (32.34 / 32.34) MB\nSuccess!\nElapsed time: 4.720 s\nDisconnected from 172.31.207.89 port 22\n```\n\nNote that the `.local` address assumes that mDNS has been configured on the\ndevice and that mDNS works on your network and OS. That's not always the case\nand a frequent source of frustration when it fails. When in doubt, check that\nyou can upload to the device's IP address. You can get the IP address from your\nrouter or by connecting to the device's IEx prompt and running `ifconfig`.","ref":"readme.html#uploading-firmware"},{"type":"extras","title":"Upload protocol - SSHSubsystemFwup","doc":"It's not necessary to use the `upload.sh` script. The following line is\nequivalent:\n\n```shell\ncat $firmware | ssh -s $nerves_device fwup\n```","ref":"readme.html#upload-protocol"},{"type":"extras","title":"Configuration - SSHSubsystemFwup","doc":"The default options should satisfy most use cases, but it's possible to alter\nhow updates are applied by passing options when creating the SSH subsystem spec\n(see `SSHSubsystemFwup.subsystem_spec/1`) or by setting the application\nenvironment.\n\nHere's an example of what the code looks like when setting options via a\nsubsystem spec:\n\n```elixir\n      :ssh.daemon(@port, [\n        ...\n        {:subsystems, [SSHSubsystemFwup.subsystem_spec(task: \"my_upgrade_task\")]}\n      ])\n```\n\nIf another library starts the SSH deamon for you, like\n[`nerves_ssh`](https://hex.pm/packages/nerves_ssh), it might be more convenient\nto set options via the application environment. `ssh_subsystem_fwup` uses its\ndefaults first, then those from the application environment and finally those in\nthe subsystem spec, so as long as the options you specify in the application\nenvironment aren't overridden, you'll be fine. Here's an example:\n\n```elixir\nconfig :ssh_subsystem_fwup, precheck_callback: {MyProject, :precheck, []}\n```\n\nThe following options are available:\n\n* `:devpath` - path for fwup to upgrade (Required)\n* `:fwup_path` - path to the fwup firmware update utility\n* `:fwup_env` - a list of name,value tuples to be passed to the OS environment for fwup\n* `:fwup_extra_options` - additional options to pass to fwup like for setting\n  public keys\n* `:precheck_callback` - an MFArgs to call when there's a connection. If\n  specified, the callback will be passed the username and the current set of\n  options. If allowed, it should return `{:ok, new_options}`. Any other return\n  value closes the connection.\n* `:success_callback` - an MFArgs to call when a firmware update completes\n  successfully. Defaults to `{Nerves.Runtime, :reboot, []}`.\n* `:task` - the task to run in the firmware update. Defaults to `\"upgrade\"`","ref":"readme.html#configuration"},{"type":"extras","title":"License - SSHSubsystemFwup","doc":"Copyright (C) 2017-21 The Nerves Project Authors  \n\n    Licensed under the Apache License, Version 2.0 (the \"License\");\n    you may not use this file except in compliance with the License.\n    You may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\n    Unless required by applicable law or agreed to in writing, software\n    distributed under the License is distributed on an \"AS IS\" BASIS,\n    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n    See the License for the specific language governing permissions and\n    limitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.6.3 - Changelog","doc":"This release adds [REUSE compliance](https://reuse.software/) to improve\ncopyright and licensing accuracy for compliance tools.\n\n* Improvements\n  * Fixed a minor Elixir 1.19 warning\n  * Removed support for Elixir 1.13 and earlier since they're no longer\n    regularly tested.","ref":"changelog.html#v0-6-3"},{"type":"extras","title":"v0.6.2 - Changelog","doc":"* Improvements\n  * Fix `mix upload` so that its invocation of `ssh` no longer overrides\n    `LD_LIBRARY_PATH`. (@ringlej)\n  * Fix/clean up some typespecs","ref":"changelog.html#v0-6-2"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"* Improvements\n  * `mix upload` now attempts to display the UUID of the firmware as well","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"* New features\n  * Added a `:precheck_callback` option to support updating firmware update\n    options at runtime and to stop updates from happening at critical times\n  * Added a `:fwup_env` option for passing OS environment variables to fwup\n  * Support setting default system-wide options in the application config in\n    addition to the subsystem spec. The subsystem spec takes precedence.","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"* Improvements\n  * Improve instructions for how to update from `nerves_firmware_ssh`","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"This releases adds a check for old `upload.sh` scripts to warn users that\nthey'll need to update it.","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"Initial release.\n\nThis factors out the SSH subsystem from\n[`nerves_firmware_ssh`](https://github.com/nerves-project/nerves_firmware_ssh)\nand removes all ssh server code. The user of this library now has to start a\nserver themselves. This makes it possible to run the firmware update on port 22\nand removes the constraint of needing to hard code authorized ssh public keys.","ref":"changelog.html#v0-5-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}